use clap::{Args, ValueEnum};
use std::{path::PathBuf, str};
use tool_helper::Error;

#[derive(Args)]
pub struct Arguments {
    #[clap(value_enum, value_parser, default_value_t=Quality::High)]
    quality: Quality,
    #[clap(value_enum, value_parser, default_value_t=Sample::Stereo)]
    sample: Sample
}

#[derive(Copy, Clone, ValueEnum)]
pub enum Quality {
    Low,
    High
}

impl Quality {
    fn get_frequency(&self) -> u64 {
        match self {
            Self::Low  => 18900,
            Self::High => 37800,
        }
    }
}

#[derive(Copy, Clone, ValueEnum)]
pub enum Sample {
    Mono,
    Stereo
}

impl Sample {
    fn get_channel(&self) -> u64 {
        match self {
            Self::Mono   => 1,
            Self::Stereo => 2,
        }
    }
}

pub fn convert(args: Arguments, input: PathBuf, output: PathBuf) -> Result<(), Error> {
    let quality   = args.quality;
    let sample    = args.sample;
    
    super::run_psxavenc(input, output, [
        "-t", "xacd", 
        "-f", quality.get_frequency().to_string().as_ref(),
        "-b", "4",
        "-c", sample.get_channel().to_string().as_ref(),
        "-F", "0",
        "-C", "0",
    ])
}